import xml.parsers.expat
from base.Logger import *

class SendResponse:

    """
        $Id: SendResponse.py,v 1.1.1.1 2003/04/10 14:38:57 maro Exp $

        Represents a response from the gateway as a reply on a sent SMS

        @author maro
    """

    RESPONSE_TYPE  = "deliver"
    STATUS_SUCCESS = "success"
    STATUS_FAIL    = "fail"

    m_raw_data = None
    m_type = None
    m_message_id = None
    m_status = None
    m_error_code = None
    m_error_msg = None
    m_current_tag = None

    m_log = None

    def __init__(self, xml=""):
        self.m_log = Logger()
        self.m_error_code = 0
        self.m_type = self.RESPONSE_TYPE
        if ( xml != "" ):
            self.m_raw_data = xml
            self.parse()

    def getRawData(self):
        """Returns the raw data representing the response in xml-format"""
        return m_raw_data

    def getMessageId(self):
        """Returns the message-id for the sent SMS"""
        return self.m_message_id

    def getErrorCode(self):
        """Returns the error-code in the response, 0=no error"""
        return self.m_error_code

    def getErrorMessage(self):
        """Returns the error-message in the response"""
        return self.m_error_msg

    def getResponseType(self):
        """Returns the response type"""
        return self.m_type

    def getStatus(self):
        """Returns status for the response"""
        if ( self.m_error_code == 0 ):
            return self.STATUS_SUCCESS
        else:
            return self.STATUS_FAIL

    def setMessageId(self, msgId):
        self.m_message_id = msgId

    def setErrorCode(self, errorCode):
        self.m_error_code = errorCode

    def setErrorMessage(self, msg):
        self.m_error_msg = msg

    def setResponseType(self, responseType):
        self.m_type = responseType

    def setStatus(self, status):
        self.m_status = status

    def toXML(self):
        """Returns an XML representation for a response"""
        xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
        xml += "<response type=\"%s\" version=\"1.0\">" % self.m_type
        if ( self.m_status == self.STATUS_FAIL ):
            xml += "<nak><error code=\"%s\">%s</error></nak>" % (self.m_error_code, self.m_error_msg)
        else:
            xml += "<ack></ack>"
        xml += "</response>"
        self.m_log.logDebug("Response: \n%s" % xml)
        return xml

    def parse(self):
        """Parses the XML-response from the gateway"""
        # Create parser
        parser = xml.parsers.expat.ParserCreate()
        parser.StartElementHandler = self.startElement
        parser.EndElementHandler = self.endElement
        parser.CharacterDataHandler = self.characters
        # Parse
        parser.Parse(self.m_raw_data)


    def characters(self, data):
        if ( self.m_current_tag == "error" ):
            self.m_error_msg = data

    def endElement(self, name):
        self.m_current_tag = None

    def startElement(self, name, atts):
        self.m_current_tag = name
        self.parseAttributes(atts)


    def parseAttributes(self, attrs):
        if ( self.m_current_tag == "response" ):
            self.m_type = attrs["type"]
        elif ( self.m_current_tag == "error" ):
            self.m_error_code = attrs["code"]
        elif ( self.m_current_tag == "message-id" ):
            self.m_message_id = attrs["value"]
