<?php

    //
    // Copyright (c) 2002-2003 Zyneo HB. (http://www.zyneo.com)
    // See LICENSE distributed with this package for more information.
    //

    include_once(_DOCROOT . "/base/Base.php");
    include_once(_DOCROOT . "/sdk/socket/ServerSocket.php");
    include_once(_DOCROOT . "/sdk/protocol/receive/Request.php");
    
    /**
     * $Id: Server.php,v 1.1.1.1 2003/03/27 10:27:42 maro Exp $
     *
     * Provides a simple socket server for receiving SMS and Notification 
     * requests from the gateway.
     *
     * @package sdk.server
     * @author Magnus Rosenquist, Zyneo
     */
    class Server
        extends Base
    {
        /** Listener object for callbacks */
        var $__listener;
        /** The listening socket */
        var $__socket;
        
        /**
         * Creates a new Server
         * @param string $host
         * @param int $port
         */
        function Server()
        {
            $this->registerClass(get_class($this));
        }
           
        /** 
         * Starts the server, which connects to the gateway and listens for
         * requests.
         */
        function run()
        {
            $this->__socket = new ServerSocket(_LISTENER_HOST, _LISTENER_PORT);
            $this->__socket->addMessageListener($this);
            $this->__socket->listen();
        }
        
        /**
         * Adds the listener which is called when requests are received.
         * This is only used by the GatewayListener.
         * @param object $listener the object listening for requests.
         * @see sdk.GatewaySender
         */     
        function addEventListener($listener)
        {
            $this->registerMethod("addEventListener");
            $this->__listener = $listener;
        }

        /** 
         * This function is called when the ServerSocket receives an incoming
         * message. The message is parsed and depending on request-type different
         * call-back methods are called, either 'onSMSReceive' or 'onNotificationReceive'.
         * @param string $message incoming from socket.
         */
        function onMessageReceive($message)
        {
            $this->registerMethod("onMessageReceive");
            $this->logDebug("Got from socket: \n" .$message."\n\n");
            $request = new Request($message);
            $type = $request->getRequestType();
            $this->logDebug("Request-type: " . $type);
            if ( $type == $request->TYPE_SMS )
            {
                $this->__listener->onSMSReceive($request->getRequest());
            }
            else if ( $type == $request->TYPE_NOT )
            {
                $this->__listener->onNotificationReceive($request->getRequest());
            }
        }        
        
        
        /**
         * Sends a Response object back to the gateway.
         * @param Response $response the response object
         */        
        function sendResponse($response)
        {
            $this->registerMethod("sendResponse");
            $msg = $response->toXML();
            $this->__socket->send($msg);
        }
        
        
    }
?>
